/***************************************************************************
    copyright            : (C) 2006 by Robby Stephenson
    email                : robby@periapsis.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of version 2 of the GNU General Public License as  *
 *   published by the Free Software Foundation;                            *
 *                                                                         *
 ***************************************************************************/

// this file gets included by tellico_config.h

public:
  static void deleteAndReset();

  static TQStringList noCapitalizationList();
  static TQStringList articleList();
  static TQStringList nameSuffixList();
  static TQStringList surnamePrefixList();

  static TQString templateName(int type);
  static TQFont templateFont(int type);
  static TQColor templateBaseColor(int type);
  static TQColor templateTextColor(int type);
  static TQColor templateHighlightedBaseColor(int type);
  static TQColor templateHighlightedTextColor(int type);

  static void setTemplateName(int type, const TQString& name);
  static void setTemplateFont(int type, const TQFont& font);
  static void setTemplateBaseColor(int type, const TQColor& color);
  static void setTemplateTextColor(int type, const TQColor& color);
  static void setTemplateHighlightedBaseColor(int type, const TQColor& color);
  static void setTemplateHighlightedTextColor(int type, const TQColor& color);
